/*
One of those Weeks Day 7
Copyright (c) 2007-2009, 2011-2012 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.inventory
{
	// Adobe Imports
	import flash.display.*;
	import flash.events.*;
	import flash.filters.GlowFilter; 

	/**
	* This is a container for holding inventory items
	*/
	public class BasicInventoryBox
	extends MovieClip
	implements IInventoryBox
	{
// ***** VARIABLES **********************************************************

		protected var _itemList:Array;
		protected var _highlightIcon:IInventoryItem;
		protected var _highlightOldFilters:Array;
		protected var _glow = new GlowFilter();
		protected var _selectedItem:IInventoryItem;
		protected var _combinedItem;
		protected var _currentInventoryState:uint;

// ***** CONSTRUCTOR ********************************************************

		public function BasicInventoryBox()
		{
			_itemList = new Array();
			addEventListener(MouseEvent.MOUSE_MOVE, handleMouseMove);
			addEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
			_highlightIcon = null;
			_currentInventoryState = InventoryConst.INVSTATE_NORMAL;
		}

// ***** Functions - Event Handlers *****************************************

		public function handleMouseMove(evt:MouseEvent)
		{
			var x:int = evt.stageX;
			var y:int = evt.stageY;
			var newHighlight:IInventoryItem = null;
			for (var cntr:int = 0; cntr < _itemList.length; ++cntr)
				if (_itemList[cntr]._iconSprite.hitTestPoint(x, y, false))
					newHighlight = _itemList[cntr];

			if (newHighlight != _highlightIcon) {
				if (_highlightIcon != null)
					_highlightIcon.setHighlight(InventoryConst.HIGHLIGHT_NONE);
				if ( (newHighlight != null) && (newHighlight != _selectedItem)){
					newHighlight.setHighlight(InventoryConst.HIGHLIGHT_OVER);
				} else
					newHighlight = null;
				_highlightIcon = newHighlight;
			}
		}

		public function handleMouseUp(evt:MouseEvent)
		{
			var clickedItem:IInventoryItem = null;
			var x:int = evt.stageX;
			var y:int = evt.stageY;
			for (var cntr:int = 0; cntr < _itemList.length; ++cntr)
				if (_itemList[cntr]._iconSprite.hitTestPoint(x, y, false))
					clickedItem = _itemList[cntr];
			if (clickedItem != null) {
				if (_currentInventoryState == InventoryConst.INVSTATE_NORMAL)
					setInventoryState(InventoryConst.INVSTATE_SELECTED, clickedItem);
				else if (_currentInventoryState == InventoryConst.INVSTATE_SELECTED)
					setInventoryState(InventoryConst.INVSTATE_COMBINED, _selectedItem, clickedItem);
				else
					setInventoryState(InventoryConst.INVSTATE_NORMAL);
			}
			_highlightIcon = null;
		}

// ***** Functions - IInventoryBox required *********************************

		/**
		* Adds the indicated item to the inventory box.
		*/
		public function addInvItem(itm:IInventoryItem)
		{
			var invItem:IInventoryItem;
			
			// make sure object not already in inventory
			for (var cntr = 0; cntr < _itemList.length; ++cntr) {
				if (invItem == itm) {
					trace ("DEBUG - Item " + itm.getItemName() + 
							" already in list");
					return;
				}
			}
			// if we reach this point, item not in inventory so add
			_itemList.push(itm);
			addChild(itm.getIcon());
			performItemLayout();
		}
		
		/**
		* removes the indicated item from the inventory box.
		*/
		public function removeInvItem(itm:IInventoryItem)
		{
			var cntr:int;
			var itmIndex:int = -1;
			
			// Array doesn't have a built in remove function so...
			// first, find the index of the item
			for (cntr = 0; cntr < _itemList.length; ++cntr)
				if (_itemList[cntr] == itm)
					itmIndex = cntr;
	
			// if item is not in list, we are done
			if (itmIndex == -1) {
				trace ("DEBUG - The item " + itm.getItemName() + 
						" is not in the inventory.");
				return;
			}
			
			// otherwise, move all items above it in list down an item
			for (cntr = itmIndex+1; cntr < _itemList.length; ++cntr)
				_itemList[cntr-1] = _itemList[cntr];
			// finally remove the last item (which is now duplicated)
			_itemList.pop();
			removeChild(itm.getIcon());
			performItemLayout();
		}
		
		/**
		* removes all the items from the inventory box.
		*/
		public function clearInventory()
		{
			var itm:IInventoryItem;
			while (_itemList.length > 0) {
				itm = _itemList.pop();
				removeChild(itm.getIcon());
			}
			performItemLayout();
		}

		/**
		* @returns array containing all items currently in the inventory box.
		*/
		public function getInventoryList():Array
		{
			return _itemList.slice();
		}

		/**
		* @return The constant integer representing the current inventory state
		*/
		public function getInventoryState():uint
		{
			return _currentInventoryState;
		}

		/**
		* Sets the state of the inventory box. This is usually done internally
		* by the box, but could be done under the programs control to allow for
		* things such as keyboard shortcuts.
		*
		* @param theState inventory state constant from InventoryConst
		* @param if the state is select, then the item to select is needed
		* @param if the state is combine, then the item to combine is needed
		*/
		public function setInventoryState(theState:uint, 
				theSelected:IInventoryItem = null,
				theSecondary:IInventoryItem = null)
		{
			// slightly inefficent but code reducing. resets state to normal
			if (_selectedItem != null) {
				_selectedItem.setHighlight(InventoryConst.HIGHLIGHT_NONE);
				_selectedItem = null;
			}
			if (_combinedItem != null) {
				_combinedItem.setHighlight(InventoryConst.HIGHLIGHT_NONE);
				_combinedItem = null;
			}
			
			// set the new state 
			switch (theState) {
			case InventoryConst.INVSTATE_NORMAL:
				// nothing needs to be done here
				break;
			case InventoryConst.INVSTATE_SELECTED:
				_selectedItem = theSelected;
				_selectedItem.setHighlight(InventoryConst.HIGHLIGHT_SELECTED);
				break;
			case InventoryConst.INVSTATE_COMBINED:
				_selectedItem = theSelected;
				_selectedItem.setHighlight(InventoryConst.HIGHLIGHT_SELECTED);
				_combinedItem = theSecondary;
				_combinedItem.setHighlight(InventoryConst.HIGHLIGHT_SELECTED);
				break;
			default:
				trace("ERROR - UNKNOWN INVENTORY BOX STATE...");
				break;
			}
			_currentInventoryState = theState;
			var evt:Event = new Event(InventoryConst.INVENTORY_EVENT);
			this.dispatchEvent(evt);
		}

		/**
		* @return the currently selected item, null if no item selected.
		*/
		public function getSelectedItem():IInventoryItem
		{
			return _selectedItem;
		}

		/**
		* @returns the item that is going to be combined with the selected
		* 		inventory item, with null being returned if none.
		*/
		public function getSecondaryItem():IInventoryItem
		{
			return _combinedItem;
		}

// ***** Functions - Internal work for handling ui ***************************

		protected function performItemLayout()
		{
			var barWidth:int = width;
			var iconWidth:int = 0;
			var cntr:int, gap:int;
			
			// calculate width and use that to determin gap width
			for (cntr = 0; cntr < _itemList.length; ++cntr)
				iconWidth += _itemList[cntr]._iconSprite.width;
			gap = (barWidth - iconWidth) / (_itemList.length + 1);
			
			// lay out the components
			var offset:int = gap;
			var itemIcon:Sprite
			for (cntr = 0; cntr < _itemList.length; ++cntr) {
				itemIcon = _itemList[cntr]._iconSprite;
				itemIcon.x = offset;
				itemIcon.y = 10;
				offset += gap;
				offset += itemIcon.width;
			}
		}
	} // end class
} // end package
